// Main
//------------------------------------------------------------------------------

// Includes
//------------------------------------------------------------------------------
#include "Worker/Worker.h"
#include "Core/Env/Assert.h"
#include "Core/Mem/MemTracker.h"

#include <windows.h>

// main
//------------------------------------------------------------------------------
#pragma warning ( push )
#pragma warning (disable:28251) // don't complain about missing annotations on WinMain
int WINAPI WinMain( HINSTANCE hInstance, 
					HINSTANCE /*hPrevInstance*/,
					LPSTR /*lpCmdLine*/, 
					int /*nCmdShow*/ )
{
	// only allow 1 worker per system
	CreateMutex( nullptr, TRUE, "Global\\FBuildWorker" );
	if ( GetLastError() == ERROR_ALREADY_EXISTS )
	{
		MessageBoxA( nullptr, "An FBuildWorker is already running!", "FBuildWorker", MB_OK );
		return -1;
	}

	// prevent popups when launching tools with missing dlls
	::SetErrorMode( SEM_FAILCRITICALERRORS );

	VERIFY( SetPriorityClass( GetCurrentProcess(), BELOW_NORMAL_PRIORITY_CLASS ) );

	// start the worker and wait for it to be closed
	int ret;
	{
		Worker worker( hInstance );
		ret = worker.Work();
	}

	MEMTRACKER_DUMP_ALLOCATIONS

	return ret;
}
#pragma warning ( pop )

//------------------------------------------------------------------------------
